require 'keybinding'
require 'physics'
require 'character_behaviour'
require 'player_behaviour'

function love.load()

	math.randomseed( os.time() )
	--key values
	TileSize = 32
	TilesetX, TilesetY = 512, 1024
	SizeX, SizeY = 200, 200
	ScreenSizeX, ScreenSizeY = 32, 20+1
	
	--scrolling
	OffsetX, OffsetY = 9,13					--IN TILE UNITS!!
	SmoothOffsetX, SmoothOffsetY = 0,0 		-- in pixels
	OffsetBorder = 8	
	Scrolling_Value = 1						--how many tiles does it jump?
	
	--charset sizes
	CharX, CharY = 32, 64
	CharFullX, CharFullY = CharX*6, CharY*4
	
	--general variables
	x,y,z,i,j,k = 0,0,0,0,0,0
	FPS = 30	
	
	MovSet = { "up", "down", "left", "right" }
	BrokenProjectiles = {}
	
	--game variables
	game_clock = 0
	
	--mouse stuff
	MouseX, MouseY, MouseTileX, MouseTileY = 0,0,1,1

	--HUD elements
	Picset = {}
	Picset = {
		love.graphics.newImage('HUD_A.png'),
	}
	
	--creating tileset
	TilesetPic = love.graphics.newImage('TILESET.png')
	Tileset = { }
	
	for j=1, TilesetX/TileSize do
		for i=1, TilesetY/TileSize do 
			Tileset[((i-1)*(TilesetX/TileSize))+j] = love.graphics.newQuad((j-1)*TileSize, (i-1)*TileSize, TileSize, TileSize, TilesetX, TilesetY)
		end
	end	
	
	--creating charsets	
	CharPic = {}	
	Charset = {}	
	
	CharPic["Robot"] = love.graphics.newImage('Charset/ROBOT.png')	
	Charset["Robot"] = {}	
	for j=1, CharFullX/CharX do
		for i=1, CharFullY/CharY do 
			Charset["Robot"][((i-1)*(CharFullX/CharX))+j] = love.graphics.newQuad((j-1)*CharX, (i-1)*CharY, CharX, CharY, CharFullX, CharFullY)
		end
	end	
	
	CharPic["Player"] = love.graphics.newImage('Charset/PLAYER.png')	
	Charset["Player"] = {}	
	for j=1, CharFullX/CharX do
		for i=1, CharFullY/CharY do 
			Charset["Player"][((i-1)*(CharFullX/CharX))+j] = love.graphics.newQuad((j-1)*CharX, (i-1)*CharY, CharX, CharY, CharFullX, CharFullY)
		end
	end		
	
	--projectiles
	CharPic["Gunshot"] = love.graphics.newImage('Charset/GUNSHOT.png')	
	Charset["Gunshot"] = {}	
	for j=1, 6 do
		for i=1, 6 do 
			Charset["Gunshot"][((i-1)*(CharFullX/CharX))+j] = love.graphics.newQuad((j-1)*TileSize, (i-1)*TileSize, TileSize, TileSize, TileSize*6, TileSize*6)
		end
	end		
	
	--uploading field
	Field = { }
	
	for j=1,SizeX do
	
		Field[j] = { }
	
		for i=1, SizeY do
			if math.random(100) < 2 then
				Field[j][i] = 18
				if i-1 ~= 0 and Field[j][i] ~= 2 then Field[j][i-1] = 2 end
			else
				Field[j][i] = 1			
			end
			
			if i==2 or j==2 or i==150 or j==150 then Field[j][i] = 2 end
		end
		
	end
	
	-- player
	Player = {
	X_Tile = OffsetX+(ScreenSizeX/2), Y_Tile = OffsetY+((ScreenSizeY-1)/2), 
	Sprite = 1,
	Is_Moving = false, Mov_Dir = 'no',
	Moving_Speed = 3,
	OffsetX = 0, OffsetY = 0,
	Inventory = {}, Handslots = {}
	}
	
	-- characters
	Characters = {}
	
	for i=1, 30 do
		Characters[i] = {
			X_Tile = 27+i, Y_Tile =  11+i*2, 
			OffsetX = 0, OffsetY = 0,				
			Sprite = 1,	
			Fraction = "Robot", Type = "Test Puppet",
			Is_Moving = false, 
			Mov_Dir = "no", Mov_Spd = 2,
			State = "Patrolling",
			
			--variables for certain states
			--this is for Patrolling
			Patrol_Direction = "right",
			Patrol_Distance = 5,
			Patrol_Storing = 0
		}
	end
	
	--projectiles
	Projectiles = {}
	

end

function love.draw()

	for j=1,ScreenSizeX do
		for i=1, ScreenSizeY do

			love.graphics.draw(TilesetPic, Tileset[Field[OffsetX+j][OffsetY+i]], 
			SmoothOffsetX+(j-1)*TileSize, 
			SmoothOffsetY+(i-1)*TileSize)

		end
	end

	--draw charcters, IF THEY ARE ON SHOWED FIELD -> they won't appear inside the HUD!
	for i=1, #Characters do
		if ((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX >= 0
		and ((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX < ScreenSizeX*TileSize
		and ((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY >= 0
		and ((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY < ScreenSizeY*TileSize then
			--the actual drawing itself
			love.graphics.draw(CharPic["Robot"],Charset["Robot"][Characters[i].Sprite], 
			((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX+SmoothOffsetX, 
			((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY+SmoothOffsetY-TileSize)
			
			--debug appear of numbers
			--love.graphics.print(Characters[i].Patrol_Storing, 
			--((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX, 
			--((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY)			
		end
	end	
	
	
	--same for projectiles
	if #Projectiles ~= 0 then
		for i=1, #Projectiles do
			if ((Projectiles[i].X_Tile-1-OffsetX)*TileSize)+Projectiles[i].OffsetX >= 0
			and ((Projectiles[i].X_Tile-1-OffsetX)*TileSize)+Projectiles[i].OffsetX < ScreenSizeX*TileSize
			and ((Projectiles[i].Y_Tile-1-OffsetY)*TileSize)+Projectiles[i].OffsetY >= 0
			and ((Projectiles[i].Y_Tile-1-OffsetY)*TileSize)+Projectiles[i].OffsetY < ScreenSizeY*TileSize then
				--the actual drawing itself
				love.graphics.draw(CharPic["Gunshot"],Charset["Gunshot"][Projectiles[i].Sprite], 
				((Projectiles[i].X_Tile-1-OffsetX)*TileSize)+Projectiles[i].OffsetX+SmoothOffsetX, 
				((Projectiles[i].Y_Tile-1-OffsetY)*TileSize)+Projectiles[i].OffsetY+SmoothOffsetY)
						
			end
		end	
	end

	
	--draw player
	love.graphics.draw(CharPic["Player"],Charset["Player"][Player.Sprite], 
	((Player.X_Tile-1-OffsetX)*TileSize)+Player.OffsetX, 
	((Player.Y_Tile-1-OffsetY)*TileSize)+Player.OffsetY-TileSize)

	love.graphics.print(Field[Player.X_Tile][Player.Y_Tile],
	((Player.X_Tile-1-OffsetX)*TileSize)+Player.OffsetX, 
	((Player.Y_Tile-OffsetY)*TileSize)+Player.OffsetY-TileSize)
	

	
	--HUD elements
	love.graphics.draw(Picset[1], 0,0)
	
	--random text
	love.graphics.print(#Projectiles.." -> "..math.floor(FPS)..", "..game_clock,0,0)
	
end

function love.update(dt)

	--key actions
	Repeated_Key_Behaviour() 
	
	--turning
	WatchMouse()
	
	--"constants"
	FPS = 1/dt
	
	MouseX, MouseY = love.mouse.getX(), love.mouse.getY()
	MouseTileX, MouseTileY = 1+math.floor((MouseX-OffsetX)/TileSize), 1+math.floor((MouseY-OffsetY)/TileSize)	
	
	--gameclock
	game_clock = game_clock + 1
	
	--move all the chars
	for i=1, #Characters do
		if not Characters[i].Is_Moving then
			Patrol_Simple(Characters[i])	
		end	

		if game_clock%10==0 then
			Characters[i].Sprite = Characters[i].Sprite +1
			if Characters[i].Sprite == 3 then Characters[i].Sprite = 1 end
		end
	
		--generate shooting
		if game_clock%30==0 then
			Create_Projectile(Characters[i], Characters[i].Mov_Dir)
		end
	end		


	--doing smooth player moving 
	if Player.Is_Moving then
		SmoothMoving_Player(Player.Mov_Dir, math.pow(2, Player.Moving_Speed))
	end

	--smooth moving of characters
	for i=1, #Characters do
		if Characters[i].Is_Moving then
			SmoothMoving_General(Characters[i], Characters[i].Mov_Dir, Characters[i].Mov_Spd)		
		end
	end
	
	for i=1, #Projectiles do

	end
	
	BrokenProjectiles = {}
	
	--smooth moving of projectiles
	for i=1, #Projectiles do

		if #Projectiles ~= 0 then	
			SmoothMoving_General(Projectiles[i], Projectiles[i].Mov_Dir, Projectiles[i].Mov_Spd)
			
			--set orientation of projectiles
			if Projectiles[i].Mov_Dir == "down" or Projectiles[i].Mov_Dir == "up" then
				Projectiles[i].Sprite = 2
			elseif Projectiles[i].Mov_Dir == "left" or Projectiles[i].Mov_Dir == "right" then
				Projectiles[i].Sprite = 1				
			else
				Projectiles[i].Sprite = 3					
			end
			
			--checks if a projectile is already hit something
			Break_Projectile(Projectiles[i], i)
		end
		
		--adds the unused projectiles to a set
		if Projectiles[i].Mov_Dir == "DELETE ME" then table.insert(BrokenProjectiles, i) end

	end	
	
	--deleting unused projectiles
	for i=#BrokenProjectiles,1, -1 do
		table.remove(Projectiles, BrokenProjectiles[i])
	end
end

--smooth moving
function SmoothMoving_Player (direction, speed)
	if (math.abs(Player.OffsetX) == TileSize or math.abs(Player.OffsetY) == TileSize)			--if either the player or
	or (math.abs(SmoothOffsetX) == TileSize or math.abs(SmoothOffsetY) == TileSize)	then		--the screen scolled a tile
		Player.Is_Moving = false																--stop moving and then
		if direction == 'up' then 																--for every directions
		
			Player.OffsetY = 0 										--change player location
			Player.Y_Tile = Player.Y_Tile - 1 
			
			if Player.Y_Tile == OffsetY + OffsetBorder 				--if player is edge of screen
			and OffsetY > 0 then									--and within boundaries
				SmoothOffsetY = 0									--then move camera one unit away
				OffsetY = OffsetY -Scrolling_Value end	
				
		elseif direction == 'down' then 
		
			Player.OffsetY = 0 
			Player.Y_Tile = Player.Y_Tile + 1
			
			if Player.Y_Tile == OffsetY + ScreenSizeY - OffsetBorder
			and OffsetY < SizeY - ScreenSizeY then 
				SmoothOffsetY = 0				
				OffsetY = OffsetY +Scrolling_Value end	
				
		elseif direction == 'left' then
		
			Player.OffsetX = 0 
			Player.X_Tile = Player.X_Tile - 1
			
			if Player.X_Tile == OffsetX + OffsetBorder 			
			and OffsetX > 0 then								
				SmoothOffsetX = 0
				OffsetX = OffsetX -Scrolling_Value end	
				
		elseif direction == 'right' then 
		
			Player.OffsetX = 0 
			Player.X_Tile = Player.X_Tile + 1
			
			if Player.X_Tile == OffsetX + ScreenSizeX - OffsetBorder 
			and OffsetX < SizeX - ScreenSizeX then 
				SmoothOffsetX = 0
				OffsetX = OffsetX +Scrolling_Value end					
		end		
	else																--if player is within a tile										
			
		if direction == 'up' then 										--then for every directions
		
			if Player.Y_Tile-1 == OffsetY + OffsetBorder 				--check if player is near screen edge
			and OffsetY > 0 then		
				SmoothOffsetY = SmoothOffsetY + speed					--if so, then scroll screen
			else
				Player.OffsetY = Player.OffsetY - speed 				--otherwise scroll player
			end
			
		elseif direction == 'down' then 
		
			if Player.Y_Tile+1 == OffsetY + ScreenSizeY - OffsetBorder 
			and OffsetY < SizeY - ScreenSizeY then 
				SmoothOffsetY = SmoothOffsetY - speed
			else
				Player.OffsetY = Player.OffsetY + speed  
			end
			
		elseif direction == 'left' then 
		
			if Player.X_Tile-1 == OffsetX + OffsetBorder 			
			and OffsetX > 0 then		
				SmoothOffsetX = SmoothOffsetX + speed
			else
				Player.OffsetX = Player.OffsetX - speed 			
			end
		

		elseif direction == 'right' then 
		
			if Player.X_Tile+1 == OffsetX + ScreenSizeX - OffsetBorder 
			and OffsetX < SizeX - ScreenSizeX
			then
				SmoothOffsetX = SmoothOffsetX - speed
			else
				Player.OffsetX = Player.OffsetX + speed 
			end
		end		


	end	

end

function SmoothMoving_General (character, direction, speed)
	if math.abs(character.OffsetX) == TileSize or math.abs(character.OffsetY) == TileSize then
		character.Is_Moving = false
		if direction == 'up' then 
			character.OffsetY = 0 
			character.Y_Tile = character.Y_Tile - 1 	
		elseif direction == 'down' then 
			character.OffsetY = 0 
			character.Y_Tile = character.Y_Tile + 1		
		elseif direction == 'left' then 
			character.OffsetX = 0 
			character.X_Tile = character.X_Tile - 1					
		elseif direction == 'right' then 
			character.OffsetX = 0 
			character.X_Tile = character.X_Tile + 1 			
		end		
	else
		if direction == 'up' then 
			character.OffsetY = character.OffsetY - speed 
		elseif direction == 'down' then 
			character.OffsetY = character.OffsetY + speed  
		elseif direction == 'left' then 
			character.OffsetX = character.OffsetX - speed 
		elseif direction == 'right' then 
			character.OffsetX = character.OffsetX + speed 
		end				
	end
	

end
































--possibly useful functions
function CountNeighbours(x,y, valueToCheck, Table)

	local asd=0

	if Table[x-1][y] == valueToCheck then asd=asd+1 end
	if Table[x+1][y] == valueToCheck then asd=asd+1 end
	if Table[x][y-1] == valueToCheck then asd=asd+1 end
	if Table[x][y+1] == valueToCheck then asd=asd+1 end	
	return asd
end

function FullNeighbours(x,y, valueToCheck, Table)
	local asd=0

	for j=-1,1 do
		for i=-1, 1 do
			if Table[x-j][y-i] == valueToCheck then asd=asd+1 end	
		end
	end
	
	return asd
end

function CountColors(Table, valueToCheck)

	local asd=0
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then asd=asd+1 end
		end
	end
	
	return asd
end

function ListColors(Table, valueToCheck)

	local list = {}
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then table.insert(list, {j,i}) end
		end
	end
	
	return list
end



function BackLoad(a,b)

	for j=1,128 do
		for i=1, 96 do
			a[j][i] = b[j][i]
		end
	end

end




